Catcrypt tools: Roll out progress indicator stuff from hashsum.
[u/mdw/catacomb] / mpmont.h
index 6b2b9bd..745cf5a 100644 (file)
--- a/mpmont.h
+++ b/mpmont.h
@@ -1,13 +1,13 @@
 /* -*-c-*-
  *
- * $Id: mpmont.h,v 1.3 1999/12/10 23:29:48 mdw Exp $
+ * $Id$
  *
  * Montgomery reduction
  *
  * (c) 1999 Straylight/Edgeware
  */
 
-/*----- Licensing notice --------------------------------------------------* 
+/*----- Licensing notice --------------------------------------------------*
  *
  * This file is part of Catacomb.
  *
  * it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
- * 
+ *
  * Catacomb is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Library General Public
  * License along with Catacomb; if not, write to the Free
  * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  * MA 02111-1307, USA.
  */
 
-/*----- Revision history --------------------------------------------------* 
- *
- * $Log: mpmont.h,v $
- * Revision 1.3  1999/12/10 23:29:48  mdw
- * Change header file guard names.
- *
- * Revision 1.2  1999/11/19 13:17:43  mdw
- * Add extra interface to exponentiation which returns a Montgomerized
- * result.  Add simultaneous exponentiation interface.
- *
- * Revision 1.1  1999/11/17 18:02:16  mdw
- * New multiprecision integer arithmetic suite.
- *
- */
-
 #ifndef CATACOMB_MPMONT_H
 #define CATACOMB_MPMONT_H
 
 
 typedef struct mpmont {
   mp *m;                               /* Modulus */
-  mpw mi;                              /* %$-m^{-1} \bmod b$% */
-  size_t shift;                                /* %$\log_2 R$% */
+  mp *mi;                              /* %$-m^{-1} \bmod R$% */
+  size_t n;                            /* %$\log_b R$% */
   mp *r, *r2;                          /* %$R \bmod m$%, %$R^2 \bmod m$% */
 } mpmont;
 
-/* --- A base/exponent pair for @mpmont_mexp@ --- */
-
-typedef struct mpmont_factor {
-  mp *base;
-  mp *exp;
-} mpmont_factor;
-
 /*----- Functions provided ------------------------------------------------*/
 
 /* --- @mpmont_create@ --- *
@@ -114,13 +92,13 @@ typedef struct mpmont_factor {
  * Arguments:  @mpmont *mm@ = pointer to Montgomery reduction context
  *             @mp *m@ = modulus to use
  *
- * Returns:    ---
+ * Returns:    Zero on success, nonzero on error.
  *
  * Use:                Initializes a Montgomery reduction context ready for use.
  *             The argument @m@ must be a positive odd integer.
  */
 
-extern void mpmont_create(mpmont */*mm*/, mp */*m*/);
+extern int mpmont_create(mpmont */*mm*/, mp */*m*/);
 
 /* --- @mpmont_destroy@ --- *
  *
@@ -163,8 +141,8 @@ extern mp *mpmont_mul(mpmont */*mm*/, mp */*d*/, mp */*a*/, mp */*b*/);
  *             @mp *a@ = base
  *             @mp *e@ = exponent
  *
- * Returns:    Result, %$a^e R \bmod m$%.  This is useful if further modular
- *             arithmetic is to be performed on the result.
+ * Returns:    Result, %$(a R^{-1})^e R \bmod m$%.  This is useful if
+ *             further modular arithmetic is to be performed on the result.
  */
 
 extern mp *mpmont_expr(mpmont */*mm*/, mp */*d*/, mp */*a*/, mp */*e*/);
@@ -185,24 +163,27 @@ extern mp *mpmont_exp(mpmont */*mm*/, mp */*d*/, mp */*a*/, mp */*e*/);
  *
  * Arguments:  @mpmont *mm@ = pointer to Montgomery reduction context
  *             @mp *d@ = fake destination
*             @mpmont_factor *f@ = pointer to array of factors
  *             @size_t n@ = number of factors supplied
  *
  * Returns:    If the bases are %$g_0, g_1, \ldots, g_{n-1}$% and the
  *             exponents are %$e_0, e_1, \ldots, e_{n-1}$% then the result
  *             is:
  *
- *             %$g_0^{e_0} g_1^{e_1} \ldots g_{n-1}^{e_{n-1}} R \bmod m$%
+ *             %$g_0^{e_0} g_1^{e_1} \ldots g_{n-1}^{e_{n-1}} \bmod m$%
+ *
+ *
+ *             except that the %$g_i$% and result are in Montgomery form.
  */
 
 extern mp *mpmont_mexpr(mpmont */*mm*/, mp */*d*/,
-                       mpmont_factor */*f*/, size_t /*n*/);
+                       const mp_expfactor */*f*/, size_t /*n*/);
 
 /* --- @mpmont_mexp@ --- *
  *
  * Arguments:  @mpmont *mm@ = pointer to Montgomery reduction context
  *             @mp *d@ = fake destination
*             @mpmont_factor *f@ = pointer to array of factors
  *             @size_t n@ = number of factors supplied
  *
  * Returns:    Product of bases raised to exponents, all mod @m@.
@@ -211,7 +192,7 @@ extern mp *mpmont_mexpr(mpmont */*mm*/, mp */*d*/,
  */
 
 extern mp *mpmont_mexp(mpmont */*mm*/, mp */*d*/,
-                      mpmont_factor */*f*/, size_t /*n*/);
+                      const mp_expfactor */*f*/, size_t /*n*/);
 
 /*----- That's all, folks -------------------------------------------------*/
@@ -185,24 +163,27 @@ extern mp *mpmont_exp(mpmont */*mm*/, mp */*d*/, mp */*a*/, mp */*e*/);
  * Arguments:  @mpmont *mm@ = pointer to Montgomery reduction context
  *             @mp *d@ = fake destination
- *             @mpmont_factor *f@ = pointer to array of factors
+ *             @const mp_expfactor *f@ = pointer to array of factors
  *             @size_t n@ = number of factors supplied
  *
@@ -185,24 +163,27 @@ extern mp *mpmont_exp(mpmont */*mm*/, mp */*d*/, mp */*a*/, mp */*e*/);
  * Arguments:  @mpmont *mm@ = pointer to Montgomery reduction context
  *             @mp *d@ = fake destination
- *             @mpmont_factor *f@ = pointer to array of factors
+ *             @const mp_expfactor *f@ = pointer to array of factors
  *             @size_t n@ = number of factors supplied
  *